/***************************************************************************
 *   Copyright (C) 2016 by Dirk Lindner <dirk.lze@gmail.com>               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "ktrans.h"

#include <QFile>
#include <QTextStream>
#include <QRegExp>
#include <QRegExp>
#include <kdebug.h>
#include <QDesktopWidget>

ktrans::ktrans(): KMainWindow()
{
    setWindowIcon( KIcon ( KIcon ("babelfish") ) );
    setWindowState(windowState() & ~Qt::WindowMinimized | Qt::WindowActive);
    setWindowFlags(Qt::WindowStaysOnTopHint);
    QWidget* w = new QWidget(this);  
    setCentralWidget(w);
    m_ui.setupUi(w);
    init();
}
ktrans::~ktrans(){}
void ktrans::closeEvent(QCloseEvent *event)
{
    hide();
    event->ignore();
}
void ktrans::init()
{
    createTray ();
    m_clip = QApplication::clipboard () ;
    connect( m_clip ,SIGNAL(changed ( QClipboard::Mode  ) ),this,SLOT(changed ( QClipboard::Mode )));
    connect( m_ui.query ,SIGNAL(returnPressed()),this,SLOT(returnPressed (  )));
    connect( m_ui.selLang ,SIGNAL(activated(int)),this,SLOT(setLang ( int )));
    
    KConfigGroup cg(KGlobal::config(), "ktrans");

    QDesktopWidget * desktop = QApplication::desktop ();
    QWidget *dw = desktop->screen();
    
    int width  = cg.readEntry("width", dw->width()/2);
    int height = cg.readEntry("height", dw->height()/2);    
    
    int posX =  cg.readEntry("posX", dw->width()/4);
    int posY =  cg.readEntry("posY",  dw->height()/4);
    
    resize ( width,height);
    move   ( posX, posY );
 
    m_sHost     = cg.readEntry("host", m_sHost);
    m_sPassword = cg.readEntry("password", m_sPassword);
    m_sUser     = cg.readEntry("username", m_sUser);
    m_sDb       = cg.readEntry("database", m_sDb);    
    m_sDing     = cg.readEntry("ding", m_sDing);    

    m_wallet = Wallet::openWallet( Wallet::LocalWallet(),winId(),Wallet::Asynchronous);
    connect(m_wallet, SIGNAL(walletOpened(bool)), SLOT(walletOpened(bool)));
    connectDb();
}
void ktrans::settings()
{
    ktransSettings* settings = new ktransSettings(m_sHost,m_sDb,m_sUser,m_sPassword,m_sDing );
    connect(settings,SIGNAL(saveSettings(QString,QString,QString,QString,QString)),this,SLOT(applySettings(QString,QString,QString,QString,QString)) );
    settings->show();
}
void ktrans::applySettings(QString host,QString db,QString user,QString pass,QString ding)
{
     m_sUser = user;
     m_sPassword = pass;
     m_sHost =  host;
     m_sDb = db;
     m_sDing = ding; 
}
void ktrans::saveSettings()
{
    QPoint pt  = pos();
    QSize sz = size();

    KConfigGroup cg(KGlobal::config(), "ktrans");
    cg.writeEntry("posX", pt.x());
    cg.writeEntry("posY", pt.y());
    cg.writeEntry("width", sz.width());
    cg.writeEntry("height", sz.height());
 
    cg.writeEntry("host", m_sHost);
    cg.writeEntry("database", m_sDb);
    cg.writeEntry("ding", m_sDing);
    cg.sync();
    QMap<QString, QString> map;
    map[QString("user")]     = m_sUser;
    map[QString("password")] = m_sPassword;
    m_wallet->writeMap("ktrans", map);
}
void ktrans::returnPressed()
{
    m_sTranslation = m_ui.query->text();
    find();
}
void ktrans::changed ( QClipboard::Mode mode ) 
{
    Q_UNUSED(mode);
    m_sTranslation = m_clip->text(mode);
    m_ui.query->text() =  m_clip->text(mode);
    if(m_bWach )
    {
      find();
    }
}
void ktrans::initDatabase () 
{
    QSqlQuery query;
    query.exec("create table translate(id int primary key, de text, en text)");     

    QFile file(m_sDing);
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
    {
      QMessageBox::critical(0, qApp->tr("Cannot open database"),
      qApp->tr("Unable to to open database File.\n"
      "This App needs MySQL support. Please read"
      "install trans-de-en"
      "to build it.\n\n"
      ) +m_sDing, QMessageBox::Cancel);
      return ;  
    }
    QTextStream in(&file);
    
    double j = 0;
    
    while (!in.atEnd())
    {
      QString line = in.readLine();
      QStringList deEnList = line.split("::");
      if(deEnList.length() == 2)
      {
	QString de  =deEnList.at(0);
	QString en  =deEnList.at(1);
	QStringList deList = de.split("|");
	QStringList enList = en.split("|");
	for(int i = 0; i< deList.length();++i)
	{
	  query.prepare("insert into translate values(?, ?, ?)");
	  query.bindValue(0, j);
	  query.bindValue(1, deList.at(i));
	  query.bindValue(2, enList.at(i));
	  query.exec();
	  j++;
	}
      }
    }
}
void ktrans::walletOpened(bool ok)
{
    if ( ok &&
    (m_wallet->hasFolder(KWallet::Wallet::FormDataFolder()) ||
    m_wallet->createFolder(KWallet::Wallet::FormDataFolder())) &&
    m_wallet->setFolder(KWallet::Wallet::FormDataFolder()))
    {
      QMap<QString, QString> map;
      m_wallet->readMap("ktrans", map);
      m_sPassword =  map[QString("password")];
      m_sUser     =  map[QString("user")];
    }
}
void ktrans::watch (bool toggled)
{
    m_bWach = toggled;
}
void ktrans::enDe (bool toggled) 
{
    m_enDeA->setChecked(toggled);
    m_deEnA->setChecked(!toggled);
    m_ui.selLang->setCurrentIndex(0);
    m_bEnDe = true; 
}
void ktrans::deEn (bool toggled) 
{
    m_enDeA->setChecked(!toggled);
    m_deEnA->setChecked(toggled);
    m_ui.selLang->setCurrentIndex(1);
    m_bEnDe = false; 
}
void ktrans::setLang(int index)
{
    Q_UNUSED(index);
    QString lng =   m_ui.selLang->currentText();
    if(lng == "English-Deutsch")
    {
      m_enDeA->setChecked(true);
      m_deEnA->setChecked(false);
      m_bEnDe = true;
    }
    else
    {
      m_enDeA->setChecked(false);
      m_deEnA->setChecked(true);
      m_bEnDe = false; 
    }
}
void ktrans::showHide(QSystemTrayIcon::ActivationReason reason)
{
    if ( reason == QSystemTrayIcon::Context)
      return;
    if(isHidden())
      showNormal();
    else
      hide ();
}
void ktrans::createTray () 
{
    m_tray = new  KSystemTrayIcon (KIcon("app-ktrans"));
    m_tray->setIcon(KIcon ("babelfish") );
    connect(m_tray, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(showHide(QSystemTrayIcon::ActivationReason))  ); 

    m_menu = new QMenu();

    m_trans  = new QAction( "Translate",NULL);
    m_trans->setIcon(KIcon ("babelfish") );
    connect(m_trans, SIGNAL(triggered()), this, SLOT(find())); 
    m_menu->addAction(m_trans);
    
    m_watchClipboard = new QAction( "watch Clipboard",NULL);    
    m_watchClipboard->setCheckable(true);
    connect(m_watchClipboard, SIGNAL(toggled(bool)), this, SLOT(watch(bool)));
    m_menu->addAction(m_watchClipboard);    

    m_deEnA = new QAction( "Deutsch-English",NULL);    
    m_deEnA->setCheckable(true);
    connect(m_deEnA, SIGNAL(toggled(bool)), this, SLOT(deEn(bool)));
    m_menu->addAction(m_deEnA);
    
    m_enDeA = new QAction( "English-Deutsch",NULL);    
    m_enDeA->setCheckable(true);
    m_enDeA->setChecked(true);
    connect(m_enDeA, SIGNAL(toggled(bool)), this, SLOT(enDe(bool)));
    m_menu->addAction(m_enDeA); 
    
    
    QAction* st = new QAction( "Settings",NULL);
    st->setIcon(  KIcon ("system-settings-symbolic")  );
    connect(st, SIGNAL(triggered()), this, SLOT(settings()));
    m_menu->addAction(st);

    QAction* quit = new QAction( "quit",NULL);
    quit->setIcon( KIcon ("application-exit")  );
    connect(quit, SIGNAL(triggered()), this, SLOT(exit()));
    m_menu->addAction(quit);
    m_tray->setContextMenu(m_menu);
    m_tray->show();
}
void ktrans::exit()
{
    saveSettings();
    qApp->quit();
}
void ktrans::connectDb()
{
    QSqlDatabase db = QSqlDatabase::addDatabase("QMYSQL");
    db.setHostName(m_sHost);
    db.setDatabaseName(m_sDb);
    db.setUserName(m_sUser);
    db.setPassword(m_sPassword);
    if (!db.open())
    {
      QMessageBox::critical(0, qApp->tr("Cannot open database"),
	qApp->tr("Unable to establish a database connection.\n"
	"This App needs MySQL support. Please read "
	"the Qt SQL driver documentation for information how "
	"to build it.\n\n"
	"Click Cancel to exit."), QMessageBox::Cancel);
    }
    else
    {
	QSqlQuery query;
	query.exec("show tables");
	bool installed = false;
	while (query.next()) 
	{
	  QString table = query.value(0).toString();
	  if(table == "translate")
	    installed = true; 
	}
	if(!installed)
	  initDatabase();
    }
}
void ktrans::find()
{
    QSqlQuery query;
    QString lng = m_bEnDe ?"en" :"de";
    query.prepare("SELECT de,en from translate where "+lng+" REGEXP ? ");
    query.bindValue(0, " "+m_sTranslation+" ");
    query.exec();
    QString table ="<table border=\"1\" width=\"100%\">";
    while (query.next()) 
    {
      QString de = query.value(0).toString();
      QString en = query.value(1).toString();
      if(m_bEnDe)
	en.replace( m_sTranslation,"<b>"+m_sTranslation+"</b>");
      else
	de.replace( m_sTranslation,"<b>"+m_sTranslation+"</b>");
      table += "<tr><td>"+de+"</td><td>"+en+"</td></tr>";
    }
    table += "</table>";
    m_ui.browser->clear();
    m_ui.browser->setText(table);
    showNormal();
}
ktransSettings::ktransSettings(QString shost,QString sdb,QString suser,QString spass,QString sding) :QDialog()
{
    setWindowIcon( KIcon ( KIcon ("babelfish") ) );
    setWindowFlags(Qt::WindowStaysOnTopHint| Qt::Dialog);
    setupUi(this);
    host->setText(shost);
    database->setText(sdb);
    user->setText(suser);
    password->setText(spass);
    ding->setText(sding);
}
ktransSettings::~ktransSettings(){}
void ktransSettings::accept()
{
  emit saveSettings(
    host->text(),
    database->text(),
    user->text(),
    password->text(),
    ding->text()
  );
}
#include "ktrans.moc"
