/***************************************************************************
 *   Copyright (C) 2016 by Dirk Lindner <dirk.lze@gmail.com>               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef KTRANS_H
#define KTRANS_H
#include <QClipboard>
#include <QSystemTrayIcon>
#include <ksystemtrayicon.h>
#include <KIcon>
#include <QMenu>
#include <QMessageBox>
#include <QSqlDatabase>
#include <QSqlQuery>
#include <QCloseEvent>
#include <QDialog>
#include <KDE/KMainWindow>
#include <KProcess>
#include "ui_ktrans.h"
#include "ui_settings.h"
#include <KWallet/Wallet>

using KWallet::Wallet;

/**
 * \mainpage ktrans is a Frontend for "ding" german-english Dictonary.
 * A German-English dictionary with ca. 270,000 entries.
 * This dictionary was designed for ding dictionary lookup program, but may be used by other clients, too.
 * Die Quelle der Datenbank ist erhältlich bei http://dict.tu-chemnitz.de/.
 * at Ubuntu it is installed per default as trans-de-en
 * ktrans alternative to ding 
 * At first start it can take a while, database with 270000 entrys will be created.
 * NEED A MySQL database to run this programm.
 * @short English German Translation via dict.tu-chemnitz.de
 * @author Dirk Lindner <dirk.lze@gmail.com>
 * @version 0.1
 */

class ktransSettings;

class ktrans : public KMainWindow
{
    Q_OBJECT
public:
    ktrans();
    virtual ~ktrans();
public slots:
    void walletOpened(bool ok);
    void changed ( QClipboard::Mode mode ) ;
    void init();
    void settings();
    void saveSettings();
    void initDatabase ();
    void createTray();
    void deEn(bool toggled);
    void enDe(bool toggled);
    void watch(bool toggled);
    void showHide(QSystemTrayIcon::ActivationReason);
    void returnPressed();
    void setLang(int index);
    void connectDb();
    void find();
    void exit();
    void applySettings(QString host,QString db,QString user,QString pass,QString ding);
protected:
  virtual void closeEvent(QCloseEvent *event);
private:
    Ui::mainWidget m_ui;
    QString m_sTranslation;
    QClipboard* m_clip;
    KSystemTrayIcon *m_tray;
    QAction* m_trans;
    QAction* m_watchClipboard;
    QAction* m_deEnA;
    QAction* m_enDeA;
    QMenu*   m_menu;
    bool m_bWach = false;
    bool m_bEnDe = true;
    QString m_sUser = "root";
    QString m_sPassword ="keinz";
    QString m_sHost = "localhost";
    QString m_sDb ="lze";
    QString m_sDing = "/usr/share/trans/de-en";
    Wallet *m_wallet;
};

class ktransSettings : public QDialog, private Ui::settings
{
     Q_OBJECT
public:
    ktransSettings(QString shost,QString sdb,QString suser,QString spass,QString sding);
    virtual ~ktransSettings();
public slots:
    void accept();
signals:
    void saveSettings(QString,QString,QString,QString,QString);
private:
    Ui::settings ui;
};

#endif // _KTRANS_H_
